@@ -58,11 +58,6 @@
 const gchar * CREATE_CURSOR_VIEW = "peek id default recipient threading hasAttachment subject status priority startDate created delivered size recurrenceKey";
 
 struct _CamelGroupwiseStorePrivate {
-	gchar *server_name;
-	gchar *user;
-
-	gchar *base_url;
-
 	GHashTable *id_hash; /*get names from ids */
 	GHashTable *name_hash;/*get ids from names */
 	GHashTable *parent_hash;
@@ -98,108 +93,26 @@ groupwise_compare_folder_name (gconstpointer a,
 	return g_str_equal (aname, bname);
 }
 
-static gboolean
-groupwise_auth_loop (CamelService *service,
-                     GCancellable *cancellable,
-                     GError **error)
-{
-	CamelSession *session = camel_service_get_session (service);
-	CamelStore *store = CAMEL_STORE (service);
-	CamelSettings *settings;
-	CamelGroupwiseStore *groupwise_store = CAMEL_GROUPWISE_STORE (store);
-	CamelGroupwiseStorePrivate *priv = groupwise_store->priv;
-	CamelNetworkSecurityMethod security_method;
-	CamelURL *url;
-	gboolean authenticated = FALSE;
-	gchar *uri;
-	guint32 prompt_flags = CAMEL_SESSION_PASSWORD_SECRET;
-	const gchar *scheme;
-	EGwConnectionErrors errors = {E_GW_CONNECTION_STATUS_INVALID_OBJECT, NULL};
-	const gchar *soap_port;
-
-	url = camel_service_get_camel_url (service);
-	settings = camel_service_get_settings (service);
-
-	security_method = camel_network_settings_get_security_method (
-		CAMEL_NETWORK_SETTINGS (settings));
-
-	soap_port = camel_groupwise_settings_get_soap_port (
-		CAMEL_GROUPWISE_SETTINGS (settings));
-
-	if (security_method == CAMEL_NETWORK_SECURITY_METHOD_NONE)
-		scheme = "http";
-	else
-		scheme = "https";
-
-	uri = g_strdup_printf (
-		"%s://%s:%s/soap", scheme, priv->server_name, soap_port);
-
-	url->passwd = NULL;
-
-	while (!authenticated) {
-
-		if (!url->passwd && !(store->flags & CAMEL_STORE_PROXY)) {
-			gchar *prompt;
-
-			prompt = camel_session_build_password_prompt (
-				"GroupWise", url->user, url->host);
-			url->passwd =
-				camel_session_get_password (
-					session, service, prompt,
-					"password", prompt_flags, error);
-			g_free (prompt);
-
-			if (!url->passwd) {
-				g_set_error (
-					error, G_IO_ERROR,
-					G_IO_ERROR_CANCELLED,
-					_("You did not enter a password."));
-				return FALSE;
-			}
-		}
-
-		priv->cnc = e_gw_connection_new_with_error_handler (uri, priv->user, url->passwd, &errors);
-
-		if (!E_IS_GW_CONNECTION (priv->cnc)) {
-			if (errors.status == E_GW_CONNECTION_STATUS_INVALID_PASSWORD) {
-				/* We need to un-cache the password before prompting again */
-				prompt_flags |= CAMEL_SESSION_PASSWORD_REPROMPT;
-				g_free (url->passwd);
-				url->passwd = NULL;
-			} else {
-				g_set_error (
-					error, CAMEL_SERVICE_ERROR,
-					CAMEL_SERVICE_ERROR_UNAVAILABLE,
-					"%s", errors.description ?
-					errors.description :
-					_("You must be working online to complete this operation"));
-				return FALSE;
-			}
-		} else
-			authenticated = TRUE;
-
-	}
-
-	return TRUE;
-}
-
 static gboolean
 check_for_connection (CamelService *service,
                       GCancellable *cancellable,
                       GError **error)
 {
-	CamelGroupwiseStore *groupwise_store = CAMEL_GROUPWISE_STORE (service);
-	CamelGroupwiseStorePrivate *priv = groupwise_store->priv;
+	CamelNetworkSettings *network_settings;
 	CamelSettings *settings;
+	const gchar *host;
 	struct addrinfo hints, *ai;
 	GError *local_error = NULL;
 
 	settings = camel_service_get_settings (service);
 
+	network_settings = CAMEL_NETWORK_SETTINGS (settings);
+	host = camel_network_settings_get_host (network_settings);
+
 	memset (&hints, 0, sizeof (hints));
 	hints.ai_socktype = SOCK_STREAM;
 	hints.ai_family = PF_UNSPEC;
-	ai = camel_getaddrinfo(priv->server_name, "groupwise", &hints, cancellable, &local_error);
+	ai = camel_getaddrinfo(host, "groupwise", &hints, cancellable, &local_error);
 	if (ai == NULL && g_error_matches (local_error, G_IO_ERROR, G_IO_ERROR_CANCELLED)) {
 		gchar *port_string;
 		const gchar *soap_port;
@@ -211,7 +124,7 @@ check_for_connection (CamelService *service,
 
 		port_string = g_strdup(soap_port);
 		ai = camel_getaddrinfo (
-			priv->server_name, port_string, &hints,
+			host, port_string, &hints,
 			cancellable, &local_error);
 		g_free (port_string);
 	}
@@ -274,7 +187,13 @@ groupwise_connect_sync (CamelService *service,
 		return TRUE;
 	}
 
-	if (!check_for_connection (service, cancellable, error) || !groupwise_auth_loop (service, cancellable, error)) {
+	if (!check_for_connection (service, cancellable, error)) {
+		camel_service_unlock (service, CAMEL_SERVICE_REC_CONNECT_LOCK);
+		camel_service_disconnect_sync (service, TRUE, NULL);
+		return FALSE;
+	}
+
+	if (!camel_session_authenticate_sync (session, service, NULL, cancellable, error)) {
 		camel_service_unlock (service, CAMEL_SERVICE_REC_CONNECT_LOCK);
 		camel_service_disconnect_sync (service, TRUE, NULL);
 		return FALSE;
@@ -325,14 +244,6 @@ groupwise_disconnect_cleanup (CamelService *service,
 	}
 
 	if (priv) {
-		if (priv->user) {
-			g_free (priv->user);
-			priv->user = NULL;
-		}
-		if (priv->server_name) {
-			g_free (priv->server_name);
-			priv->server_name = NULL;
-		}
 		if (priv->port) {
 			g_free (priv->port);
 			priv->port = NULL;
@@ -341,10 +252,6 @@ groupwise_disconnect_cleanup (CamelService *service,
 			g_free (priv->use_ssl);
 			priv->use_ssl = NULL;
 		}
-		if (priv->base_url) {
-			g_free (priv->base_url);
-			priv->base_url = NULL;
-		}
 
 		if (groupwise_store->root_container)
 			g_free (groupwise_store->root_container);
@@ -634,7 +541,7 @@ groupwise_store_get_folder_sync (CamelStore *store,
 			cancellable,
 			_("Fetching summary information for new messages in %s"),
 			camel_folder_get_display_name (folder));
-		camel_folder_summary_clear (folder->summary);
+		camel_folder_summary_clear (folder->summary, NULL);
 
 		while (!done) {
 				status = e_gw_connection_read_cursor (priv->cnc, container_id,
@@ -739,7 +646,7 @@ gw_store_reload_folder (CamelGroupwiseStore *gw_store,
 	}
 
 	summary = (CamelGroupwiseSummary *) folder->summary;
-	camel_folder_summary_clear (folder->summary);
+	camel_folder_summary_clear (folder->summary, NULL);
 	camel_folder_summary_save_to_db (folder->summary, NULL);
 
 	summary_count = camel_folder_summary_count (folder->summary);
@@ -1340,17 +1247,23 @@ gchar *
 groupwise_get_name (CamelService *service,
                     gboolean brief)
 {
-	CamelURL *url;
+	CamelNetworkSettings *network_settings;
+	CamelSettings *settings;
+	const gchar *host;
+	const gchar *user;
 
-	url = camel_service_get_camel_url (service);
+	settings = camel_service_get_settings (service);
+
+	network_settings = CAMEL_NETWORK_SETTINGS (settings);
+	host = camel_network_settings_get_host (network_settings);
+	user = camel_network_settings_get_user (network_settings);
 
 	if (brief)
 		return g_strdup_printf (
-			_("GroupWise server %s"), url->host);
+			_("GroupWise server %s"), host);
 	else
 		return g_strdup_printf (
-			_("GroupWise service for %s on %s"),
-			url->user, url->host);
+			_("GroupWise service for %s on %s"), user, host);
 }
 
 const gchar *
@@ -1377,12 +1290,6 @@ cnc_lookup (CamelGroupwiseStorePrivate *priv)
 	return priv->cnc;
 }
 
-const gchar *
-groupwise_base_url_lookup (CamelGroupwiseStorePrivate *priv)
-{
-	return priv->base_url;
-}
-
 static CamelFolder *
 groupwise_store_get_trash_folder_sync (CamelStore *store,
                                        GCancellable *cancellable,
@@ -1512,9 +1419,6 @@ groupwise_store_finalize (GObject *object)
 
 	groupwise_store = CAMEL_GROUPWISE_STORE (object);
 
-	g_free (groupwise_store->priv->user);
-	g_free (groupwise_store->priv->server_name);
-	g_free (groupwise_store->priv->base_url);
 	g_free (groupwise_store->root_container);
 
 	if (groupwise_store->priv->id_hash != NULL)
@@ -1530,6 +1434,77 @@ groupwise_store_finalize (GObject *object)
 	G_OBJECT_CLASS (camel_groupwise_store_parent_class)->finalize (object);
 }
 
+static CamelAuthenticationResult
+groupwise_store_authenticate_sync (CamelService *service,
+                                   const gchar *mechanism,
+                                   GCancellable *cancellable,
+                                   GError **error)
+{
+	CamelStore *store = CAMEL_STORE (service);
+	CamelGroupwiseStore *groupwise_store = CAMEL_GROUPWISE_STORE (store);
+	CamelGroupwiseStorePrivate *priv = groupwise_store->priv;
+	CamelNetworkSettings *network_settings;
+	CamelNetworkSecurityMethod method;
+	CamelAuthenticationResult result;
+	CamelSettings *settings;
+	gchar *uri;
+	const gchar *host;
+	const gchar *user;
+	const gchar *scheme;
+	const gchar *password;
+	EGwConnectionErrors errors = {E_GW_CONNECTION_STATUS_INVALID_OBJECT, NULL};
+	const gchar *soap_port;
+
+	password = camel_service_get_password (service);
+	settings = camel_service_get_settings (service);
+
+	if (password == NULL) {
+		g_set_error_literal (
+			error, CAMEL_SERVICE_ERROR,
+			CAMEL_SERVICE_ERROR_CANT_AUTHENTICATE,
+			_("Authentication password not available"));
+		return CAMEL_AUTHENTICATION_ERROR;
+	}
+
+	network_settings = CAMEL_NETWORK_SETTINGS (settings);
+	host = camel_network_settings_get_host (network_settings);
+	user = camel_network_settings_get_user (network_settings);
+	method = camel_network_settings_get_security_method (network_settings);
+
+	soap_port = camel_groupwise_settings_get_soap_port (
+		CAMEL_GROUPWISE_SETTINGS (settings));
+
+	if (method == CAMEL_NETWORK_SECURITY_METHOD_NONE)
+		scheme = "http";
+	else
+		scheme = "https";
+
+	uri = g_strdup_printf (
+		"%s://%s:%s/soap", scheme, host, soap_port);
+
+	priv->cnc = e_gw_connection_new_with_error_handler (
+		uri, user, password, &errors);
+
+	g_free (uri);
+
+	if (E_IS_GW_CONNECTION (priv->cnc)) {
+		result = CAMEL_AUTHENTICATION_ACCEPTED;
+	} else if (errors.status == E_GW_CONNECTION_STATUS_INVALID_PASSWORD) {
+		result = CAMEL_AUTHENTICATION_REJECTED;
+	} else {
+		g_set_error (
+			error, CAMEL_SERVICE_ERROR,
+			CAMEL_SERVICE_ERROR_UNAVAILABLE,
+			"%s", errors.description ?
+			errors.description :
+			_("You must be working online "
+			  "to complete this operation"));
+		result = CAMEL_AUTHENTICATION_ERROR;
+	}
+
+	return result;
+}
+
 static gboolean
 groupwise_store_initable_init (GInitable *initable,
                                GCancellable *cancellable,
@@ -1539,11 +1514,9 @@ groupwise_store_initable_init (GInitable *initable,
 	CamelService *transport;
 	CamelService *service;
 	CamelSession *session;
-	CamelURL *url;
 	const gchar *user_data_dir;
 	const gchar *store_uid;
 	gchar *transport_uid;
-	gchar *uri_string;
 	gchar *path = NULL;
 
 	groupwise_store = CAMEL_GROUPWISE_STORE (initable);
@@ -1553,17 +1526,9 @@ groupwise_store_initable_init (GInitable *initable,
 		return FALSE;
 
 	service = CAMEL_SERVICE (initable);
-	url = camel_service_get_camel_url (service);
 	session = camel_service_get_session (service);
 	user_data_dir = camel_service_get_user_data_dir (service);
 
-	if (!(url->host || url->user)) {
-		g_set_error (
-			error, CAMEL_SERVICE_ERROR,
-			CAMEL_SERVICE_ERROR_INVALID,
-			_("Host or user not available in url"));
-	}
-
 	/*store summary*/
 	path = g_alloca (strlen (user_data_dir) + 32);
 	sprintf (path, "%s/.summary", user_data_dir);
@@ -1572,15 +1537,6 @@ groupwise_store_initable_init (GInitable *initable,
 	camel_store_summary_touch ((CamelStoreSummary *) groupwise_store->summary);
 	camel_store_summary_load ((CamelStoreSummary *) groupwise_store->summary);
 
-	/*host and user*/
-	groupwise_store->priv->server_name = g_strdup (url->host);
-	groupwise_store->priv->user = g_strdup (url->user);
-
-	/*base url*/
-	groupwise_store->priv->base_url = camel_url_to_string (
-		url, CAMEL_URL_HIDE_PASSWORD |
-		CAMEL_URL_HIDE_PARAMS | CAMEL_URL_HIDE_AUTH);
-
 	/*Hash Table*/
 	groupwise_store->priv->id_hash =
 		g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free);
@@ -1596,19 +1552,25 @@ groupwise_store_initable_init (GInitable *initable,
 
 	store_uid = camel_service_get_uid (service);
 	transport_uid = g_strconcat (store_uid, "-transport", NULL);
-	uri_string = camel_url_to_string (url, CAMEL_URL_HIDE_ALL);
 
 	transport = camel_session_add_service (
-		session, transport_uid, uri_string,
+		session, transport_uid, "groupwise",
 		CAMEL_PROVIDER_TRANSPORT, error);
 
 	g_free (transport_uid);
-	g_free (uri_string);
 
-	if (transport != NULL)
+	if (transport != NULL) {
+		/* Share settings with the store. */
+		g_object_bind_property (
+			groupwise_store, "settings",
+			transport, "settings",
+			G_BINDING_BIDIRECTIONAL |
+			G_BINDING_SYNC_CREATE);
+
 		camel_groupwise_transport_set_store (
 			CAMEL_GROUPWISE_TRANSPORT (transport),
 			groupwise_store);
+	}
 
 	return (transport != NULL);
 }
@@ -1631,6 +1593,7 @@ camel_groupwise_store_class_init (CamelGroupwiseStoreClass *class)
 	service_class->get_name = groupwise_get_name;
 	service_class->connect_sync = groupwise_connect_sync;
 	service_class->disconnect_sync = groupwise_disconnect_sync;
+	service_class->authenticate_sync = groupwise_store_authenticate_sync;
 	service_class->query_auth_types_sync = groupwise_store_query_auth_types_sync;
 
 	store_class = CAMEL_STORE_CLASS (class);
@@ -1662,8 +1625,6 @@ camel_groupwise_store_init (CamelGroupwiseStore *groupwise_store)
 		CamelGroupwiseStorePrivate);
 
 	d("in groupwise store init\n");
-	groupwise_store->priv->server_name = NULL;
-	groupwise_store->priv->user = NULL;
 	groupwise_store->priv->cnc = NULL;
 	groupwise_store->current_folder = NULL;
 }
