@@ -424,11 +424,14 @@ proxy_login_add_new_store (CamelStore *store,
 	EShell *shell;
 	EShellBackend *shell_backend;
 	EMailBackend *mail_backend;
+	EMailSession *mail_session;
 	gint permissions = GPOINTER_TO_INT(g_object_get_data ((GObject *)account, "permissions"));
 
 	shell = e_shell_get_default ();
 	shell_backend = e_shell_get_backend_by_name (shell, "mail");
+
 	mail_backend = E_MAIL_BACKEND (shell_backend);
+	mail_session = e_mail_backend_get_session (mail_backend);
 
 	if (store == NULL)
 		return;
@@ -437,7 +440,7 @@ proxy_login_add_new_store (CamelStore *store,
 		store->mode &= !CAMEL_STORE_WRITE;
 
 	store->flags |= CAMEL_STORE_PROXY;
-	e_mail_store_add (mail_backend, store);
+	e_mail_store_add (mail_session, store);
 }
 
 static void
